@@ -1,7 +1,7 @@
 #!/usr/bin/python
 
 """
-	Squeezecontrol: Control your Squeezeboxes from your N900
+	Squeezecontrol: Control your Squeezeboxes from your N900.
 
 	Copyright 2011 Mikkel Munch Mortensen <3xm@detfalskested.dk> 
 
@@ -146,7 +146,7 @@ class AboutWindow(QMainWindow):
 	def render(self):
 		container = QWidget()
 		vbox = QVBoxLayout(container)
-		label = QLabel('This version of Squeezecontrol was made while listening to Mux Mool, DJ Shadow and Looptroop. A huge shoutout to JingleManSweep who made the pysqueezecenter wrapper library for Python.\n\nCopyright 2010, 2011 JingleManSweep and Mikkel Munch Mortensen. Released under the terms of the GNU General Public License version 3.')
+		label = QLabel('This version of Squeezecontrol was made while listening to Daft Punk. A huge shoutout to JingleManSweep who made the pysqueezecenter wrapper library for Python.\n\nCopyright 2010, 2011 JingleManSweep and Mikkel Munch Mortensen. Released under the terms of the GNU General Public License version 3.')
 		label.setWordWrap(True)
 		vbox.addWidget(label)
 		
@@ -158,45 +158,71 @@ class SettingsWindow(QMainWindow):
 		self.setAttribute(Qt.WA_Maemo5StackedWindow)
 		self.setWindowTitle('Settings')
 		self.main = parent
+		self.fields = {}
 		self.render()
 	
 	def render(self):
 		container = QWidget(self)
+		container.setMinimumSize(792,0)
 		vbox = QVBoxLayout(container)
+		vbox.setSpacing(25)
 		
-		label = QLabel('Set the Squeezebox Server name. E.g. an IP address or something like myservername.com')
+		label = QLabel('For Squeezeconnect to work, it needs to know how to connect to your Squeezebox Server. Please fill out the fields below to match your server setup.')
 		label.setWordWrap(True)
-		#label.setAlignment(Qt.AlignCenter)
+
 		vbox.addWidget(label)
 		
-		fields = QWidget()
-		hbox = QHBoxLayout(fields)
-		vbox.addWidget(fields)
-		
-		label = QLabel('Server name:')
-		hbox.addWidget(label)
-		
-		self.server_field = QLineEdit()
-		server = self.main.get_setting('server')
-		server = server if server else ''
-		self.server_field.setText(server)
-		hbox.addWidget(self.server_field)
+		vbox.addWidget(self.render_field('server', 'Server name', 'E.g. an IP address or something like myservername.com'))
+		vbox.addWidget(self.render_field('port', 'Server CLI port', 'Optional. Leave blank if you use the default port (9090).', validator = QIntValidator()))
+		vbox.addWidget(self.render_field('username', 'Username', 'Optional. Leave blank if you haven\'t enabled server authenticion.'))
+		vbox.addWidget(self.render_field('password', 'Password', 'Optional. Leave blank if you haven\'t enabled server authenticion.', echo_mode = QLineEdit.EchoMode.Password))
 		
 		button = QPushButton('Save')
 		self.connect(button, SIGNAL('clicked()'), self.save_settings)
-		hbox.addWidget(button)
+		vbox.addWidget(button)
 		
-		label = QLabel('Is your server running on a non-default port? Are you using username/password to autheticate to your server? Have patience. These options will be added in a future version (coming soon).')
-		label.setWordWrap(True)
-		#label.setAlignment(Qt.AlignCenter)
-		vbox.addWidget(label)
+		scroller = QScrollArea()
+		scroller.setWidget(container)
+		self.setCentralWidget(scroller)
+	
+	def render_field(self, option_name, label_text, help_text, validator = None, echo_mode = None):
+		vwidget = QWidget()
+		vbox = QVBoxLayout(vwidget)
+		vbox.setContentsMargins(0, 0, 0, 0)
+		vbox.setSpacing(0)
+				
+		hwidget = QWidget()
+		hbox = QHBoxLayout(hwidget)
+		hbox.setContentsMargins(0, 0, 0, 0)
 		
-		self.setCentralWidget(container)
+		label = QLabel(label_text)
+		label.setFixedWidth(160)
+		hbox.addWidget(label)
+		
+		field = QLineEdit()
+		if validator:
+			field.setValidator(validator)
+		if echo_mode:
+			field.setEchoMode(echo_mode)
+		option = self.main.get_setting(option_name)
+		option = option if option else ''
+		field.setText(str(option))
+		hbox.addWidget(field)
+		self.fields[option_name] = field
+		
+		vbox.addWidget(hwidget)
+		
+		help = QLabel(help_text)
+		help.setAlignment(Qt.AlignRight)
+		vbox.addWidget(help)
+		
+		return vwidget
 	
 	def save_settings(self):
-		self.main.save_setting('server', self.server_field.text())
+		for setting in ['server', 'port', 'username', 'password']:
+			self.main.save_setting(setting, self.fields[setting].text())
 		info = QMaemo5InformationBox()
-		info.information(self, 'Server name saved.', 2500)
+		info.information(self, 'Settings saved.', 2500)
 
 class SelectPlayerWindow(QMainWindow):
 	def __init__(self, parent):
@@ -206,28 +232,37 @@ class SelectPlayerWindow(QMainWindow):
 		self.render()
 	
 	def render(self):
-		server = self.main.get_setting('server')
-		if server:
-			self.render_player_list(server)
+		settings = {}
+		for value in ['server', 'port', 'username', 'password']:
+			settings[value] = self.main.get_setting(value)
+		if settings['server']:
+			self.render_player_list(settings)
 		else:
 			self.setWindowTitle('No server set')
 			container = QWidget(self)
 			vbox = QVBoxLayout(container)
 			label = QLabel('Please go back to the main screen, go to "Settings" and set what server to use.')
 			label.setWordWrap(True)
-			#label.setAlignment(Qt.AlignCenter)
 			vbox.addWidget(label)
 			self.setCentralWidget(container)
 		
-	def render_player_list(self, server):
+	def render_player_list(self, settings):
 		try:
-			self.sc = Server(hostname=server)
+			settings['port'] = int(settings['port'])
+		except:
+			settings['port'] = 9090
+		try:
+			if not settings['port']:
+				settings['port'] = 9090
+			if settings['username']:
+				self.sc = Server(hostname=settings['server'], port=settings['port'], username=str(settings['username']), password=str(settings['password']))
+			else:
+				self.sc = Server(hostname=settings['server'], port=settings['port'])
 			self.sc.connect()
 		except Exception:
 			self.setWindowTitle('Connection error')
-			label = QLabel('Unable to connect to Squeezebox server at %s. Is the server accessible via your current network? Did you enter the right server settings?' % server)
+			label = QLabel('Unable to connect to Squeezebox server at %s. Is the server accessible via your current network? Did you enter the right server settings?' % settings['server'])
 			label.setWordWrap(True)
-			#label.setAlignment(Qt.AlignCenter)
 			self.setCentralWidget(label)
 		else:
 			self.setWindowTitle('Select player')
